
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <unistd.h>
#include <getopt.h>
#include <strophe.h>


#define USER0 "nagios0.tester"
#define PASS0 "muc"

#define USER1 "nagios1.tester"
#define PASS1 "muc"


/* http://nagiosplug.sourceforge.net/developer-guidelines.html */

int verbose_flag=0;
int success = 0;
char *version="1.0";
char *error_message = NULL;

void
add_to_error_message (char *str)
{
  if (error_message)
    {
      char *nem = (char *) malloc (strlen (error_message) + strlen (str) + 10);
      sprintf (nem, "%s%s", error_message, str);
      free (error_message);
      error_message = nem;
    }
  else
    error_message = strdup (str);
}


typedef enum
  {
    client_state_connecting,
    client_state_connected,
    client_state_in_muc,
    client_state_waiting_for_child,

    client_state_saw_child_talk,
    client_state_waiting_for_parent,
    client_state_done,

    client_state_error
  } client_state;


char *
state_to_string (client_state state)
{
  static char *st_to_str[] =
    {
      "client_state_connecting",
      "client_state_connected",
      "client_state_in_muc",
      "client_state_waiting_for_child",

      "client_state_saw_child_talk",
      "client_state_waiting_for_parent",
      "client_state_done",

      "client_state_error"
    };

  return st_to_str [state];
}


typedef struct context_struct
{
  client_state state;
  int sent_ok_count;
  time_t last_talked;
  char *name;
  char *last_message_received;

  xmpp_ctx_t *ctx;
  xmpp_conn_t *conn;
  char *sl_username;
  char *sl_password;
  char *xmpp_server;
  char *jid_domain;
  char *resource;
  char *muc_name;
  char *muc_jid;
  char *jid;

  char *test_string;
  char *test_string_done;

} context;



char *random_string (char *dst, int size)
{
  static const char text[] =
    "abcdefghijklmnopqrstuvwxyz"
    "ABCDEFGHIJKLMNOPQRSTUVWXYZ";

  int i;

  for (i = 0; i < size-1; ++i)
    {
      dst[ i ] = text[ rand () % (sizeof text - 1) ];
    }
  dst[ i ] = '\0';

  return dst;
}


int presence_handler (xmpp_conn_t *const conn,
                      xmpp_stanza_t *const stanza,
                      void *const userdata)
{
  context *c = (context *) userdata;
  char *from;

  from = (char *) malloc (strlen (c->muc_name) +
                          strlen (c->jid_domain) +
                          strlen (c->sl_username) + 40);
  sprintf (from, "%s@conference.%s/%s",
           c->muc_name, c->jid_domain, c->sl_username);

  if (strcmp (xmpp_stanza_get_attribute (stanza, "to"), c->jid) == 0 &&
      strcmp (xmpp_stanza_get_attribute (stanza, "from"), from) == 0)
    {
      if (c->state == client_state_connected)
        c->state = client_state_in_muc;
      else
        fprintf (stderr, "entered muc while not in connected state?\n");
    }

  free (from);

  if (verbose_flag)
    {
      fprintf (stderr,
               "%s in presence_handler, to='%s', from='%s', in-muc=%d\n",
               c->name,
               xmpp_stanza_get_attribute (stanza, "to"),
               xmpp_stanza_get_attribute (stanza, "from"),
               (c->state == client_state_in_muc));
    }

  return 1;
}


int error_handler (xmpp_conn_t *const conn,
                   xmpp_stanza_t *const stanza,
                   void *const userdata)
{
  context *c = (context *) userdata;
  fprintf (stderr, "%s in error_handler\n", c->name);
  c->state = client_state_error;
  return 1;
}


int
version_handler (xmpp_conn_t * const conn,
                 xmpp_stanza_t * const stanza, void * const userdata)
{
  xmpp_stanza_t *reply, *query, *name, *version, *text;
  char *ns;
  context *c = (context *) userdata;
  xmpp_ctx_t *ctx = c->ctx;

  if (verbose_flag)
    fprintf (stderr, "%s received version request from %s\n",
             c->name,
             xmpp_stanza_get_attribute (stanza, "from"));

  reply = xmpp_stanza_new (ctx);
  xmpp_stanza_set_name (reply, "iq");
  xmpp_stanza_set_type (reply, "result");
  xmpp_stanza_set_id (reply, xmpp_stanza_get_id (stanza));
  xmpp_stanza_set_attribute (reply, "to",
                             xmpp_stanza_get_attribute (stanza, "from"));

  query = xmpp_stanza_new (ctx);
  xmpp_stanza_set_name (query, "query");
  ns = xmpp_stanza_get_ns (xmpp_stanza_get_children (stanza));
  if (ns)
    {
      xmpp_stanza_set_ns (query, ns);
    }

  name = xmpp_stanza_new (ctx);
  xmpp_stanza_set_name (name, "name");
  xmpp_stanza_add_child (query, name);

  text = xmpp_stanza_new (ctx);
  xmpp_stanza_set_text (text, "nagios check");
  xmpp_stanza_add_child (name, text);

  version = xmpp_stanza_new (ctx);
  xmpp_stanza_set_name (version, "version");
  xmpp_stanza_add_child (query, version);

  text = xmpp_stanza_new (ctx);
  xmpp_stanza_set_text (text, "1.0");
  xmpp_stanza_add_child (version, text);

  xmpp_stanza_add_child (reply, query);

  xmpp_send (conn, reply);
  xmpp_stanza_release (reply);
  return 1;
}


void say (context *c, char *message)
{
  xmpp_stanza_t *msg_stanza, *body, *nick, *text;

  if (verbose_flag)
    fprintf (stderr, "%s sending '%s'\n", c->name, message);

  msg_stanza = xmpp_stanza_new (c->ctx);
  xmpp_stanza_set_name (msg_stanza, "message");
  xmpp_stanza_set_type (msg_stanza, "groupchat");

  xmpp_stanza_set_attribute (msg_stanza, "to", c->muc_jid);
  /* xmpp_stanza_set_attribute (msg_stanza, "from", c->jid); */

  body = xmpp_stanza_new (c->ctx);
  xmpp_stanza_set_name (body, "body");
  text = xmpp_stanza_new (c->ctx);
  xmpp_stanza_set_text (text, message);
  xmpp_stanza_add_child (body, text);
  xmpp_stanza_add_child (msg_stanza, body);

  nick = xmpp_stanza_new (c->ctx);
  xmpp_stanza_set_name (nick, "nick");
  xmpp_stanza_set_attribute (nick, "xmlns", "http://jabber.org/protocol/nick");
  text = xmpp_stanza_new (c->ctx);
  xmpp_stanza_set_text (text, c->sl_username);
  xmpp_stanza_add_child (nick, text);
  xmpp_stanza_add_child (msg_stanza, nick);

  xmpp_send (c->conn, msg_stanza);
  xmpp_stanza_release (msg_stanza);
}



char *receive (context *c, xmpp_stanza_t *const stanza)
{
  char *intext;

  if (!xmpp_stanza_get_child_by_name (stanza, "body"))
    return NULL;
  if (!strcmp (xmpp_stanza_get_attribute (stanza, "type"), "error"))
    {
      if (verbose_flag)
        fprintf (stderr, "%s got error message from %s\n",
                 c->name,
                 xmpp_stanza_get_attribute (stanza, "from"));
      c->state = client_state_error;
      return NULL;
    }
  if (xmpp_stanza_get_child_by_name (stanza, "delay"))
    return NULL;

  intext = xmpp_stanza_get_text (xmpp_stanza_get_child_by_name (stanza, "body"));
  if (verbose_flag)
    fprintf (stderr, "%s got incoming message from %s: '%s'\n",
             c->name,
             xmpp_stanza_get_attribute (stanza, "from"), intext);

  return intext;
}


int message_handler (xmpp_conn_t *const conn,
                     xmpp_stanza_t *const stanza,
                     void *const userdata)
{
  context *c = (context *) userdata;
  char *intext = receive (c, stanza);

  if (intext == NULL)
    {
      if (verbose_flag)
        {
          fprintf (stderr, "%s got NULL message?\n", c->name);
        }
      return 1;
    }

  if (c->last_message_received)
    free (c->last_message_received);
  c->last_message_received = strdup (intext);

  return 1;
}



void conn_handler (xmpp_conn_t * const conn, const xmpp_conn_event_t status,
                   const int error, xmpp_stream_error_t * const stream_error,
                   void * const userdata)
{
  context *c = (context *) userdata;
  xmpp_ctx_t *ctx = c->ctx;

  if (status == XMPP_CONN_CONNECT)
    {
      if (verbose_flag)
        fprintf (stderr, "%s: connected\n", c->name);
      xmpp_handler_add (conn,version_handler,
                        "jabber:iq:version", "iq", NULL, c);

      /* Send initial <presence/> so that we appear online to contacts */
      {
        xmpp_stanza_t* pres;
        pres = xmpp_stanza_new (ctx);
        xmpp_stanza_set_name (pres, "presence");
        xmpp_send (conn, pres);
        xmpp_stanza_release (pres);
      }

      /* http://xmpp.org/extensions/xep-0045.html#enter-muc */
      /*
        <presence to="nagios-test-group@conference.chat.aditi.lindenlab.com/seth.horsefly" >
        <priority>5</priority>
        <c xmlns="http://jabber.org/protocol/caps" node="http://psi-im.org/caps" ver="caps-b75d8d2b25" ext="ca cs ep-notify html" />
        <x xmlns="http://jabber.org/protocol/muc"/>
        </presence>
      */

      {
        char *from, *to;
        xmpp_stanza_t* pres;
        pres = xmpp_stanza_new (ctx);
        xmpp_stanza_set_name (pres, "presence");

        from = (char *) malloc (strlen (c->sl_username) +
                                strlen (c->jid_domain) + 10);
        sprintf (from, "%s@%s", c->sl_username, c->jid_domain);

        to = (char *) malloc (strlen (c->muc_name) +
                              strlen (c->jid_domain) +
                              strlen (c->sl_username) + 40);
        sprintf (to, "%s@conference.%s/%s",
                 c->muc_name, c->jid_domain, c->sl_username);

        xmpp_stanza_set_attribute (pres, "from", from);
        xmpp_stanza_set_attribute (pres, "to", to);

        free (from);
        free (to);
        xmpp_send (conn, pres);
        xmpp_stanza_release (pres);
      }

      if (c->state == client_state_connecting)
        c->state = client_state_connected;
      else
        fprintf (stderr, "connected while not connecting?\n");
    }
  else
    {
      if (verbose_flag)
        fprintf (stderr, "%s disconnected\n", c->name);
      add_to_error_message (c->name);
      add_to_error_message (" -- no connection.  ");
      xmpp_stop (ctx);
      c->state = client_state_error;
    }
}



void
request_room_ownership (context *c)
{
  xmpp_stanza_t *stanza, *query_stanza, *x_stanza;

  if (verbose_flag)
    fprintf (stderr, "%s sending request for room ownership\n", c->name);

  /*
<iq type="set" to="h.p.-hovercraft-co@conference.chat.aditi.lindenlab.com" id="aabfa" >
<query xmlns="http://jabber.org/protocol/muc#owner">
<x xmlns="jabber:x:data" type="submit" />
</query>
</iq>
  */

  /*
<iq to="nagios-test-group@conference.chat.aditi.lindenlab.com"><query xmlns="http://jabber.org/protocol/muc#owner"><x xmlns="jabber:x:data" type="submit"/></query></iq>
  */

  stanza = xmpp_stanza_new (c->ctx);
  xmpp_stanza_set_name (stanza, "iq");
  xmpp_stanza_set_attribute (stanza, "to", c->muc_jid);
  xmpp_stanza_set_type (stanza, "set");

  query_stanza = xmpp_stanza_new (c->ctx);
  xmpp_stanza_set_name (query_stanza, "query");
  xmpp_stanza_set_attribute (query_stanza, "xmlns",
                             "http://jabber.org/protocol/muc#owner");

  x_stanza = xmpp_stanza_new (c->ctx);
  xmpp_stanza_set_name (x_stanza, "x");
  xmpp_stanza_set_attribute (x_stanza, "xmlns", "jabber:x:data");
  xmpp_stanza_set_type (x_stanza, "submit");

  xmpp_stanza_add_child (stanza, query_stanza);
  xmpp_stanza_add_child (query_stanza, x_stanza);

  xmpp_send (c->conn, stanza);
  xmpp_stanza_release (stanza);
}


void
change_nickname (context *c)
{
  /*
    <presence
        from='hag66@shakespeare.lit/pda'
        to='darkcave@chat.shakespeare.lit/oldhag'/>
  */

  xmpp_stanza_t *stanza;

  if (verbose_flag)
    fprintf (stderr, "%s sending request for room list\n", c->name);

  stanza = xmpp_stanza_new (c->ctx);
  xmpp_stanza_set_name (stanza, "presence");
  xmpp_stanza_set_attribute (stanza, "from", c->jid);
  xmpp_stanza_set_attribute (stanza, "to", c->jid);

  xmpp_send (c->conn, stanza);
  xmpp_stanza_release (stanza);
}


void ready_connection (context *c,
                       char *username, char *password, char *encoded_group_name,
                       char *jid_domain, char *xmpp_server)
{
  xmpp_log_t *log;
  xmpp_conn_t *conn;
  char *jid, *muc_jid;

  memset (c, 0, sizeof (context));

  c->state = client_state_connecting;
  c->sl_username = strdup (username);
  c->sl_password = strdup (password);
  c->muc_name = strdup (encoded_group_name);
  c->jid_domain = strdup (jid_domain);
  c->xmpp_server = strdup (xmpp_server);
  c->resource = strdup ("nagios");

  xmpp_initialize ();

  if (verbose_flag > 1)
    log = xmpp_get_default_logger (XMPP_LEVEL_DEBUG);
  else
    log = xmpp_get_default_logger (XMPP_LEVEL_ERROR);

  c->ctx = xmpp_ctx_new (NULL, log);

  jid = (char *) malloc (strlen (c->sl_username) +
                         strlen (c->jid_domain) +
                         strlen (c->resource) + 10);
  sprintf (jid, "%s@%s/%s", c->sl_username, c->jid_domain, c->resource);
  c->jid = jid;

  muc_jid = (char *) malloc (strlen (encoded_group_name) +
                             strlen (c->jid_domain) + 30);
  sprintf (muc_jid, "%s@conference.%s", encoded_group_name, c->jid_domain);
  c->muc_jid = muc_jid;

  conn = xmpp_conn_new (c->ctx);
  xmpp_conn_set_jid (conn, jid);
  xmpp_conn_set_pass (conn, c->sl_password);

  c->conn = conn;
}


void end_connection (context *c)
{
  xmpp_conn_release (c->conn);
  xmpp_ctx_free (c->ctx);
  xmpp_shutdown ();
}


void client_parent (char *username, char *password, char *encoded_group_name,
                    char *jid_domain, char *xmpp_server,
                    char *test_string, char *test_string_done)
{
  int tries;
  context c;
  char name[ 1024 ];
  int attempts = 0;

 again:
  attempts ++;
  if (attempts > 5)
    return;

  ready_connection (&c, username, password, encoded_group_name,
                    jid_domain, xmpp_server);

  c.test_string = strdup (test_string);
  c.test_string_done = strdup (test_string_done);

  snprintf (name, sizeof (name), "PARENT (%s)", xmpp_server);
  c.name = strdup (name);

  xmpp_connect_client (c.conn, c.xmpp_server, 5222, conn_handler, &c);
  xmpp_handler_add (c.conn, message_handler, NULL, "message", NULL, &c);
  xmpp_handler_add (c.conn, presence_handler, NULL, "presence", NULL, &c);
  xmpp_handler_add (c.conn, error_handler, NULL, "error", NULL, &c);

  for (tries=0; tries<200; tries++)
    {
      if (verbose_flag)
        fprintf (stderr, "state=%s%s%s\n",
                 "\x1b[32m",
                 state_to_string (c.state),
                 "\x1b[0m"
                 );

      if (c.state == client_state_done)
        {
          if (verbose_flag)
            fprintf (stderr, "success.\n");
          success = 1;
          break;
        }

      xmpp_run_once (c.ctx, 1000);

      switch (c.state)
        {
        case client_state_error:
          end_connection (&c);
          c.state = client_state_connecting;
          goto again;
        case client_state_connecting:
          break;
        case client_state_connected:
          break;
        case client_state_in_muc:
          /* here, if the parent tries to talk in the channel, we can
             tell if it's really in the room, or if we got caught in
             a room creation race: http://www.ejabberd.im/node/4585
             if we aren't actually in the room, we'll get an error
             response back, which will cause a re-connection.
          */
          c.state = client_state_waiting_for_child;
          break;
        case client_state_waiting_for_child:
          if (c.last_message_received &&
              strcmp (c.last_message_received, c.test_string) == 0)
            c.state = client_state_saw_child_talk;
          break;
        case client_state_saw_child_talk:
          say (&c, c.test_string_done);
          c.state = client_state_waiting_for_parent;
          break;
        case client_state_waiting_for_parent:
          if (c.last_message_received &&
              strcmp (c.last_message_received, c.test_string_done) == 0)
            c.state = client_state_done;
          break;
        default:
          fprintf (stderr, "in an unexpected state: %d\n", c.state);
          exit (1);
        }
    }

  end_connection (&c);
}


void client_child (char *username, char *password, char *encoded_group_name,
                   char *jid_domain, char *xmpp_server,
                   char *test_string, char *test_string_done)
{
  int tries;
  context c;
  char name[ 1024 ];
  int attempts = 0;

 again:
  attempts ++;
  if (attempts > 5)
    return;

  /*
    connect, send a test string a few times.  if this client sees the "done"
    string, it exits.  it gives up after 20 test strings.
  */

  ready_connection (&c, username, password, encoded_group_name,
                    jid_domain, xmpp_server);

  c.test_string = strdup (test_string);
  c.test_string_done = strdup (test_string_done);

  snprintf (name, sizeof (name), "CHILD (%s)", xmpp_server);
  c.name = strdup (name);

  xmpp_connect_client (c.conn, c.xmpp_server, 5222, conn_handler, &c);
  xmpp_handler_add (c.conn, message_handler, NULL, "message", NULL, &c);
  xmpp_handler_add (c.conn, presence_handler, NULL, "presence", NULL, &c);

  /* 60 seconds at most */
  for (tries=0; tries<60; tries++)
    {
      if (verbose_flag)
        fprintf (stderr, "state=%s\n", state_to_string (c.state));

      xmpp_run_once (c.ctx, 1000);

      if (c.state == client_state_done)
        break;

      switch (c.state)
        {
        case client_state_error:
          end_connection (&c);
          c.state = client_state_connecting;
          goto again;
        case client_state_connecting:
          break;
        case client_state_connected:
          break;
        case client_state_in_muc:
          {
            time_t now = time (NULL);

            if (c.last_message_received &&
                strcmp (c.last_message_received, c.test_string_done) == 0)
              {
                if (verbose_flag)
                  fprintf (stderr, "%s got 'done'\n", c.name);
                c.state = client_state_done;
                break;
              }

            if (now - c.last_talked > 1)
              {
                if (c.sent_ok_count == 20)
                  {
                    c.state = client_state_done;
                    break;
                  }

                c.sent_ok_count ++;
                c.last_talked = now;
                say (&c, test_string);
              }
            break;
          }
        case client_state_done:
          break;
        default:
          fprintf (stderr, "in an unexpected state: %d\n", c.state);
          exit (1);
        }
    }

  end_connection (&c);
}


void usage (int argc, char **argv)
{
  fprintf (stderr,
           "%s [arguments]\n"
           " -V --version\n"
           " -h --help\n"
           " -t --timeout <seconds>>\n"
           " -w --warning <seconds>\n"
           " -c --critical <seconds>\n"
           " -H --hostname <hostname>, use 1 or 2 times\n"
           " -j --jid-domain <xmpp jid domain>\n"
           " -f --first-host-only\n"
           " -s --second-host-only\n"
           " -g --group-name <encoded group name>\n"
           " -r --race\n"
           " -v --verbose (use twice for more)\n\n", argv[ 0 ]);

  fprintf (stderr,
           "example:\n"
           "  xmpp-nagios-check -H chat-dfw0.aditi.lindenlab.com "
           "-H chat-dfw1.aditi.lindenlab.com -j chat.aditi.lindenlab.com\n"
           "\n"
           "\n"
           "This check:\n"
           "(1) connects to (at most) two xmpp servers\n"
           "(2) joins the same MUC room on both\n"
           "(3) sends some messages from one connection\n"
           "(4) determines if they can be seen on the other\n\n");

    fprintf (stderr,
           "non-nagios options are:\n"
           "\n"
           "-j     jid hostname -- doesn\'t have to match actual hostname(s).\n"
           "-f     only make the \"listening\" connection.\n"
           "-s     only make the \"talking\" connection.\n"
           "-g     the name of the MUC room to join\n"
           "-r     without this, the parent sleeps for 1 second.  see:\n"
           "       http://www.ejabberd.im/node/4585\n"
           "-v     debugging output; use twice for more.\n");

  exit (1);
}


int main (int argc, char **argv)
{
  int start_time, duration;
  int fork_return = 0;
  int c;
  int first_host_only = 0;
  int second_host_only = 0;
  float timeout = 0;
  float warning = 10;
  float critical = 20;
  int race_flag = 0;
  char *hostname0=NULL, *hostname1=NULL;
  char *jid_domain = "chat.aditi.lindenlab.com";
  char *group_name = "nagios-test-group";
  char test_string[ 20 ];
  char test_string_done[ 40 ];

  while (1)
    {
      static struct option long_options[] =
        {
          {"version",  no_argument,       0, 'V'},
          {"help",     no_argument,       0, 'h'},
          {"timeout",  required_argument, 0, 't'},
          {"warning",  required_argument, 0, 'w'},
          {"critical",  required_argument, 0, 'c'},
          {"hostname",  required_argument, 0, 'H'},
          {"jid-domain",  required_argument, 0, 'j'},
          {"first-host-only",  required_argument, 0, 'f'},
          {"second-host-only",  required_argument, 0, 's'},
          {"verbose", no_argument, 0, 'v'},
          {"race", no_argument, 0, 'r'},
          {0, 0, 0, 0}
        };

      /* getopt_long stores the option index here. */
      int option_index = 0;
     
      c = getopt_long (argc, argv, "Vht:w:c:H:j:fsvr?",
                       long_options, &option_index);
     
      /* Detect the end of the options. */
      if (c == -1)
        break;
     
      switch (c)
        {
        case 0:
          /* If this option set a flag, do nothing else now. */
          if (long_options[ option_index ].flag != 0)
            break;
          printf ("option %s", long_options[ option_index ].name);
          if (optarg)
            printf (" with arg %s", optarg);
          printf ("\n");
          break;
     
        case 'V':
          printf ("xmpp-nagios-check %s\n", version);
          exit (0);
     
        case 'h':
          usage (argc, argv);

        case 'f':
          first_host_only = 1;
          break;

        case 's':
          second_host_only = 1;
          break;
     
        case 't':
          sscanf (optarg, "%f", &timeout);
          break;
     
        case 'w':
          sscanf (optarg, "%f", &warning);
          break;
     
        case 'c':
          sscanf (optarg, "%f", &critical);
          break;

        case 'H':
          if (hostname0)
            hostname1 = strdup (optarg);
          else if (hostname0)
            {
              fprintf (stderr, "Use -H (--hostname) 1 or 2 times.\n");
              usage (argc, argv);
            }
          else
            hostname0 = strdup (optarg);
          break;

        case 'j':
          jid_domain = strdup (optarg);
          break;

        case '?':
          /* getopt_long already printed an error message. */
          usage (argc, argv);

        case 'v':
          verbose_flag ++;
          break;

        case 'r':
          race_flag = 1;
          break;
     
        default:
          abort ();
        }
    }

  if (hostname1 == NULL)
    {
      if (hostname0 == NULL)
        {
          fprintf (stderr, "Use -H (--hostname) 1 or 2 times.\n");
          usage (argc, argv);
        }
      hostname1 = strdup (hostname0);
    }

  if (first_host_only && second_host_only)
    {
      fprintf (stderr, "don't use both -f and -s.\n");
      usage (argc, argv);
    }


  random_string (test_string, 20);
  sprintf (test_string_done, "%s -- done", test_string);

  if (verbose_flag)
    {
      fprintf (stderr, "test_string='%s'\n", test_string);
      fprintf (stderr, "test_string_done='%s'\n", test_string_done);
    }

  start_time = time (NULL);


  if ((first_host_only == 0) && (second_host_only == 0))
    {
      fork_return = fork ();
      if (fork_return < 0)
        {
          perror ("fork");
          return (1);
        }
    }

  if ((first_host_only || fork_return) && (second_host_only == 0))
    {
      /* parent */
      if (race_flag == 0)
        sleep (2);
      client_parent (USER0, PASS0,
                     group_name /*"nagios-test-group" */,
                     jid_domain /*"chat.aditi.lindenlab.com" */,
                     hostname0, /* "chat-dfw2.aditi.lindenlab.com" */
                     test_string, test_string_done);
    }
  else
    {
      /* child */
      /* close (0); close (1); close (2); */
      
      client_child (USER1, PASS1,
                    group_name /*"nagios-test-group" */,
                    jid_domain /* "chat.aditi.lindenlab.com" */,
                    hostname1, /* "chat-dfw2.aditi.lindenlab.com" */
                    test_string, test_string_done);
      return 0;
    }


  /* 0=normal, 1=warning, 2=critical, 3=unknown */
  if (success)
    {
      duration =  time (NULL) - start_time;
      if (duration < warning)
        {
          printf ("success after %d seconds\n", duration);
          return 0;
        }
      if (duration < critical)
        {
          printf ("success/warning after %d seconds\n", duration);
          return 1;
        }
      printf ("success/critical after %d seconds\n", duration);
      return 2;
    }

  if (error_message)
    printf ("%s\n", error_message);
  else
    fprintf (stderr, "parent got to end with no error_message?\n");
  return 3;
}
